// Based on RODial from:
// Copyright (C) 2012 Ron Ostafichuk
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

/* SVN FILE: $Id: widget_current.php 455 2013-07-07 19:54:58Z anders $
* Project Name : ThermIQ*
* @version $Revision: 455 $
* @lastrevision $Date: 2013-07-07 21:54:58 +0200 (s, 07 jul 2013) $
* @modifiedby $LastChangedBy: anders $
* @lastmodified $LastChangedDate: 2013-07-07 21:54:58 +0200 (s, 07 jul 2013) $
*/



// function to draw alarm arcs
function DrawAlarmArc(ctx,xCenter_px,yCenter_px,sColorOut,sColorIn,min_rad,max_rad,nRadius_px,nArcThickness_px)
{
	// draw the alarm range arc with an outside color and an inside arc to simulate a gradient (some browsers can not do proper gradients yet)
	var nOuterRadius_px = nRadius_px + nArcThickness_px/2;
	var nInnerRadius_px = nRadius_px - nArcThickness_px/2;
	
	ctx.fillStyle = sColorOut;
	ctx.save();
	ctx.beginPath();
	ctx.moveTo(xCenter_px+Math.cos(min_rad)*nInnerRadius_px, yCenter_px+Math.sin(min_rad)*nInnerRadius_px);
	ctx.arc(xCenter_px, yCenter_px, nOuterRadius_px, min_rad, max_rad, false);
	ctx.lineTo(xCenter_px+Math.cos(max_rad)*nInnerRadius_px, yCenter_px+Math.sin(max_rad)*nInnerRadius_px);
	ctx.arc(xCenter_px, yCenter_px, nInnerRadius_px, max_rad, min_rad, true);
	ctx.closePath();
	ctx.fill();
	ctx.restore();

	// draw an inner arc using the color sColorIn
	var nOuterRadius_px = nRadius_px + nArcThickness_px/4;
	var nInnerRadius_px = nRadius_px - nArcThickness_px/4;
	
	ctx.fillStyle = sColorIn;
	ctx.save();
	ctx.beginPath();
	ctx.moveTo(xCenter_px+Math.cos(min_rad)*nInnerRadius_px, yCenter_px+Math.sin(min_rad)*nInnerRadius_px);
	ctx.arc(xCenter_px, yCenter_px, nOuterRadius_px, min_rad, max_rad, false);
	ctx.lineTo(xCenter_px+Math.cos(max_rad)*nInnerRadius_px, yCenter_px+Math.sin(max_rad)*nInnerRadius_px);
	ctx.arc(xCenter_px, yCenter_px, nInnerRadius_px, max_rad, min_rad, true);
	ctx.closePath();
	ctx.fill();
	ctx.restore();
}

// Function to draw an html5 dial type Gauge using the canvas tag
/*
 * drawDialGauge params:
 * sControlID = DOM id for canvas element (canvas should have dimensions width=height for best results)
 * sParams = semicolon delimited string containing
 *  title= text string for top middle of gauge
 *  arrow= 1 - long skinny triangle pointer
 *         2 - rectangle with white tip and highlights
 *         3 - rectangle with triangle at end
 *  min= integer, lowest value to show on gauge
 *  ll= integer, start of the low emergency level
 *  l= integer, start of the low caution level
 *  h= integer, start of the high caution
 *  hh= integer, start of the high emergency level
 *  max= integer, highest value to show on gauge
 *  val= integer, needle point level
 *  display= friendly text for display at bottom of gauge
 *  display2= optional text for second line of display at bottom of gauge
 */

function drawDialGauge(sControlID, sParams) {
	var canvas = document.getElementById(sControlID);
	if (!canvas.getContext) {
		$('#' + sControlID).html('Unsupported: ' + sParams);
		return; // this browser does not support the canvas (html5)
	}
	var ctx = canvas.getContext('2d');
	
	// default inputs for debug, similar to json format
	if (sParams.length <= 0)
		sParams = "arrow=1;title=Sample;min=0;ll=100;l=500;h=2500;hh=3500;max=5000;val=50;display=50 kPa;display2=Pressure;background-color=rgba(255,255,255,0)"; 
	var aLabels = []; // used to put labels on image with x,y,l and optional text color c
	var NA = -9e9; // not available
	var nMin = NA, nMax = NA, nL = NA, nLL = NA, nH = NA, nHH = NA, nReading = NA;
	var sDisplay = ""; // text to be displayed for the gauge reading
	var sTitle = ""; // title for the gauge
	var nArrow = 1;
	var aParams = sParams.split(";"); // param format key=val;

	// basic colors need for drawing
	var sBlack = '#000000';
	var sWhite = '#FFFFFF';
	var sGray1 = '#E1E1E1', sGray2 = '#A4A4A4', sGray3 = '#919191', sGray4= '#515151';
	var sBlue1 = '#a1a1FF', sBlue2 = '#6565FF', sBlue = '#0000FF';
	var sBlue6 = '#0000DF', sBlue7 = '#000050';
	var sLightRed = '#FF6161', sRed = '#EE0000', sDarkRed = '#B00000';
	var sDarkYellow='#DaDa00',sYellow = '#FaFa00', sLightYellow = '#FaFa60';
	var sGreen = '#00FF00', sLightGreen = '#a0ffa0';
	var sOrange = '#FFA500';

	var sBackgroundColor = ''; // transparent background if string is empty

	// get all needed params from the input string
	for ( var i = 0; i < aParams.length; i++) {
		var aKeyVal = aParams[i].split("=");
		var sVal = aKeyVal[1];
		var nVal = parseFloat(sVal);
		switch( aKeyVal[0] )
		{
			case "arrow":
				if( nVal != NaN )
					nArrow = nVal;
				break;
			case "title":
				sTitle = sVal;
				break;
			case "min":
				if( nVal != NaN )
					nMin = nVal;
				break;
			case "max":
				if( nVal != NaN )
					nMax = nVal;
				break;
			case "l":
				if( nVal != NaN )
					nL = nVal;
				break;
			case "ll":
				if( nVal != NaN )
					nLL = nVal;
				break;
			case "h":
				if( nVal != NaN )
					nH = nVal;
				break;
			case "hh":
				if( nVal != NaN )
					nHH = nVal;
				break;
			case "val":
				nReading = nVal;
				break;
			case "display":
				sDisplay = sVal;
				break;
			case "display2":
				sDisplay2 = sVal;
				break;
			case "background-color":
				sBackgroundColor = sVal;
				break;
			default:
		}			
	}
	// prevent bad values
	if (nL <= nMin)
		nL = NA;
	if (nLL <= nMin)
		nLL = NA;
	if (nH <= nMin)
		nH = NA;
	if (nHH <= nMin)
		nHH = NA;

	// get some critical dimensions for drawing
	var nWidth = ctx.canvas.width;
	var nHalfWidth = nWidth/2;
	var nHeight = ctx.canvas.height;
	if (nHeight>nWidth) nHeight=nWidth;

	// figure out the font size to use
	var sFont = "14px 'arial'";
	var nFontHeight = Math.floor(nWidth * 14 / 240)+1; // estimate based on 14px
	if (nFontHeight < 7)
		nFontHeight = 7;
	if (nFontHeight > 28)
		nFontHeight = 28;
	var nMinFontHeight = nFontHeight / 2;
	if (nMinFontHeight < 8)
		nMinFontHeight = 8;
	var sFont = nFontHeight + "px 'arial'";
	var sMinFont = nMinFontHeight + "px 'arial'";

	ctx.font = sFont;

	// calc dimensions for drawing
	var nBorder = 3;
	var minRadius = nHalfWidth - nBorder;
	var maxRadius = nHalfWidth - nBorder;
	if (nHeight < nWidth)
		minRadius = nHalfWidth - nBorder;
	if (nHeight > nWidth)
		maxRadius = nHalfWidth - nBorder;

	// calc center for plot
	var xc = nHalfWidth;
	var yc = nHeight / 2;

	// clear the gauge area
	ctx.fillstyle = sBackgroundColor;
	ctx.lineCap = "square"; // most browsers only support square linecaps
	if( sBackgroundColor.length > 0 )
		ctx.fillRect(0, 0, nWidth, nHeight); // clear the gauge area according to user requested rgba
	else
		ctx.clearRect(0,0,nWidth,nHeight);
	// calc the start and end of the arcs for the dial in radians
	var arcStart_rad, arcEnd_rad;
	arcStart_rad = 130 / 180 * Math.PI;
	arcEnd_rad = 50 / 180 * Math.PI;

	// calc the position of the line and the yellow and red indicators on the
	// gauge
	totalArc_rad = arcEnd_rad - arcStart_rad;
	if (arcEnd_rad < arcStart_rad)
		totalArc_rad += 2 * Math.PI; // wrap around
	if (nReading < nMin) 
	{
		nReading = nMin; // do not allow arrow to go below the minimum gauge position!
	}
	if (nReading > nMax) 
	{
		var nOrder = 0.0000001;
		for ( var nOrder = 0.00000001; nOrder < 1000000000; nOrder *= 10) 
		{
			if (nReading > nOrder)
				nMax = nOrder * (Math.floor(nReading / nOrder) + 1); 
			// prevent reading from being greater than the max
		}
	}
	var nRange = nMax-nMin;
	
	// default params as not avalable
	var redMin_rad = NA, yellowMin_rad = NA, yellowMax_rad = NA, redMax_rad = NA;
	// get the angle in radians for each parameter
	if (nLL > NA)
		redMin_rad = nLL * totalArc_rad / nRange + arcStart_rad;
	if (nL > NA)
		yellowMin_rad = nL * totalArc_rad / nRange + arcStart_rad;
	if (nH > NA)
		yellowMax_rad = nH * totalArc_rad / nRange + arcStart_rad;
	if (nHH > NA)
		redMax_rad = nHH * totalArc_rad / nRange + arcStart_rad;

	arrow_rad = nReading * totalArc_rad / nRange + arcStart_rad;

	// Set gauge face color
	var sTextColorForReading = sBlack;
	var sColorOfGaugeFace = sWhite;
	var nHighlightRadius = nCenterPinRadius;
	// set up flashing to indicate alarm 
	if ((nReading <= nL && nReading > nLL) || (nReading >= nH && nReading < nHH && nHH > NA))
		sColorOfGaugeFace = sYellow;
	if (nReading <= nLL || (nReading >= nHH && nHH > NA))
		sColorOfGaugeFace = sRed;

	// draw body of gauge
	nOutsideRingThickness = nWidth / 40;

	var nDotRadius = nOutsideRingThickness;
	// make an outside ring
	// MSIE and chrome does not support gradient, so do grey rings
	var nColor = 170;
	var nColorStep = Math.floor((255 - 170) / nOutsideRingThickness);
	for ( var n = nHalfWidth; n > nHalfWidth - nOutsideRingThickness; n = n - 1) {
		ctx.fillStyle = "rgba(" + nColor + ", " + nColor + ", " + nColor
				+ ", 1)";
		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.moveTo(xc, yc);
		ctx.arc(xc, yc, n, 0, 2 * Math.PI, 0);
		ctx.lineTo(xc, yc);
		ctx.fill();
		ctx.restore();
		if (n > minRadius)
			nColor += nColorStep;
		else
			nColor -= nColorStep;
	}

	ctx.fillStyle = '#ffffff';
	ctx.save();
	ctx.lineWidth = 1;
	ctx.beginPath();
	ctx.moveTo(xc, yc);
	ctx.arc(xc, yc, nHalfWidth - nOutsideRingThickness - 1, 0, 2 * Math.PI,
			0);
	ctx.lineTo(xc, yc);
	ctx.fill();
	ctx.restore();

	// outside edge of the arcs for the color indicators
	var nArcWidthForColors = maxRadius * 2 - 5 * nOutsideRingThickness; 
	var nArcHeightForColors = maxRadius * 2 - 5 * nOutsideRingThickness;

	var nMaxTickSize = nWidth / 10;
	// inside white arc area
	var nInsideHeight = nArcWidthForColors - 1.8 * nOutsideRingThickness; 
	var nOutsideOfGaugeLine = nInsideHeight - nMaxTickSize / 5.5
			- nOutsideRingThickness;
	var nGaugeLineWidth = nMaxTickSize / 3;
	var nCenterPinRadius = nInsideHeight / 20;

	ctx.lineWidth = 0.5;
	ctx.strokeStyle = sGray2;
	var nHalfWidth = nOutsideRingThickness*0.5;

	if (nMin < nL && nL < nMax) {
		// make sure red min exists
		if( redMin_rad == NA )
			redMin_rad = arcStart_rad;
		// yellow min part of ring
		DrawAlarmArc(ctx,xc,yc,sDarkYellow,sYellow,redMin_rad,yellowMin_rad,nArcWidthForColors / 2,nOutsideRingThickness);
	}
	if (nMin < nLL && nLL < nMax && redMin_rad != arcStart_rad) {
		// red min part of ring
		DrawAlarmArc(ctx,xc,yc,sDarkRed,sRed,arcStart_rad,redMin_rad,nArcWidthForColors / 2,nOutsideRingThickness);
	}
	// yellow max part of ring
	if (nMin < nH && nH < nMax) {
		// make sure redMax exists
		if( redMax_rad == NA )
			redMax_rad = arcEnd_rad;
		DrawAlarmArc(ctx,xc,yc,sDarkYellow,sYellow,yellowMax_rad,redMax_rad,nArcWidthForColors / 2,nOutsideRingThickness);
	}
	// red max part of ring
	if (nMin < nHH && nHH < nMax && redMax_rad != arcEnd_rad) {
		DrawAlarmArc(ctx,xc,yc,sDarkRed,sRed,redMax_rad,arcEnd_rad,nArcWidthForColors / 2,nOutsideRingThickness);
	}
	
	// gauge arc for the ticks to be drawn on top of
	ctx.fillStyle = sGray3;

	// outside dark part
	ctx.save();
	var nHalfWidth = nOutsideRingThickness / 3;
	ctx.lineWidth = 1;
	ctx.beginPath();
	ctx.arc(xc, yc, nOutsideOfGaugeLine / 2 + nHalfWidth, arcStart_rad,
			arcEnd_rad, 0);
	ctx.lineTo(xc, yc);
	ctx.closePath();
	ctx.fill();
	ctx.restore();
	// inside lighter highlight
	ctx.fillStyle = sGray1;
	ctx.save();
	ctx.lineWidth = 1;
	ctx.beginPath();
	ctx.arc(xc, yc, nOutsideOfGaugeLine / 2 + nHalfWidth / 2, arcStart_rad,
			arcEnd_rad, 0);
	ctx.lineTo(xc, yc);
	ctx.closePath();
	ctx.fill();
	ctx.restore();

	ctx.save();
	ctx.fillStyle = sWhite;
	ctx.beginPath();
	ctx.arc(xc, yc, nOutsideOfGaugeLine / 2 - nHalfWidth, 0, Math.PI * 2, 1);
	ctx.lineTo(xc, yc);
	ctx.closePath();
	ctx.fill();
	ctx.restore();
	
	// draw ticks and labels for the gauge readings
	// calculate the number of major and minor ticks (6 major ticks, 5
	// divisions)
	var nBestMajorDiv = Math.ceil(nRange / 5);
	var numMajorDivs = nRange / nBestMajorDiv;
	// limit to a reasonable range
	while (numMajorDivs > 10.0)
		numMajorDivs /= 2;
	while (numMajorDivs < 4.0)
		numMajorDivs *= 2;

	var numMinorDivsPerMajor = 4.0; // default

	majorTickStep_rad = totalArc_rad / numMajorDivs;
	minorTickStep_rad = majorTickStep_rad / numMinorDivsPerMajor;
	// draw small ticks only
	var iLoop = 0;
	for ( var nStep_rad = 0; nStep_rad < totalArc_rad; nStep_rad += minorTickStep_rad) {
		if (iLoop++ % numMinorDivsPerMajor == 0)
			continue; // skip this tick
		// draw small ticks first as small rectangles
		// calculate the position of the ticks
		var tick_rad = arcStart_rad + nStep_rad;
		var cosOfTick = Math.cos(tick_rad);
		var sinOfTick = Math.sin(tick_rad);
		var x1 = xc + cosOfTick * (nInsideHeight) / 2.0; // just inside the
		// color sections
		var y1 = yc + sinOfTick * (nInsideHeight) / 2.0;
		var x2 = xc + cosOfTick * (nInsideHeight - nMaxTickSize * 0.8) / 2.0;
		var y2 = yc + sinOfTick * (nInsideHeight - nMaxTickSize * 0.8) / 2.0;
		// calc ? pixel perpendicular offset to be able to draw rectangles
		tickPerp_rad = tick_rad - 90.0 / 180 * Math.PI;
		var xPerp = Math.cos(tickPerp_rad) * nMaxTickSize / 10;
		var yPerp = Math.sin(tickPerp_rad) * nMaxTickSize / 10;

		ctx.fillStyle = sGray1;
		ctx.strokeStyle = sGray2;

		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.moveTo(x1 - xPerp, y1 - yPerp);
		ctx.lineTo(x2, y2);
		ctx.lineTo(x2, y2);
		ctx.lineTo(x1 + xPerp, y1 + yPerp);
		ctx.closePath();
		ctx.fill();
		ctx.stroke();
		ctx.restore();
	}
	// draw big ticks thicker and with a text label
	for (nStep_rad = 0; nStep_rad <= totalArc_rad + minorTickStep_rad; nStep_rad += majorTickStep_rad) {
		// calculate the position of the ticks and the 4 corners of a rectangle
		var tick_rad = arcStart_rad + nStep_rad;
		var cosOfTick = Math.cos(tick_rad);
		var sinOfTick = Math.sin(tick_rad);
		var x1 = xc + cosOfTick * (nInsideHeight + nMaxTickSize / 8) / 2.0; 
		// just inside the color sections
		var y1 = yc + sinOfTick * (nInsideHeight + nMaxTickSize / 8) / 2.0;
		var x2 = xc + cosOfTick * (nInsideHeight - nMaxTickSize) / 2.0;
		var y2 = yc + sinOfTick * (nInsideHeight - nMaxTickSize) / 2.0;
		// calc ? pixel perpendicular offset to be able to draw rectangles
		tickPerp_rad = tick_rad - 90.0 / 180 * Math.PI;
		var xPerp = Math.cos(tickPerp_rad) * nMaxTickSize / 10;
		var yPerp = Math.sin(tickPerp_rad) * nMaxTickSize / 10;

		var xPerp2 = Math.cos(tickPerp_rad) * nMaxTickSize / 10;
		var yPerp2 = Math.sin(tickPerp_rad) * nMaxTickSize / 10;

		ctx.fillStyle = sGray1;
		ctx.strokeStyle = sGray2;

		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.moveTo(x1 - xPerp, y1 - yPerp);
		ctx.lineTo(x2 - xPerp2, y2 - yPerp2);
		ctx.lineTo(x2 + xPerp2, y2 + yPerp2);
		ctx.lineTo(x1 + xPerp, y1 + yPerp);
		ctx.closePath();
		ctx.fill();
		ctx.stroke();
		ctx.restore();

		// calc center position of text label
		x1 = xc + cosOfTick * (nInsideHeight - nMaxTickSize * 2.5) / 2.0; // just inside the ticks
		y1 = yc + sinOfTick * (nInsideHeight - nMaxTickSize * 2.5) / 2.0;

		sLabel = Math.floor(nStep_rad * nRange / totalArc_rad + 0.5)
				+ '';
		var nLabelWidth = 15;
		if (ctx.measureText)
			nLabelWidth = ctx.measureText(sLabel).width;
		if (nLabelWidth <= nFontHeight)
			nLabelWidth = Math.max(1, sLabel.length) * nFontHeight * 0.7;
		aLabels.push({
			x : x1 - nLabelWidth / 2,
			y : y1 + nFontHeight / 4,
			l : sLabel
		});
	}

	nArrowLen = (nInsideHeight - nMaxTickSize / 2) / 2.0;
	xTip = xc + Math.cos((arrow_rad)) * nArrowLen;
	yTip = yc + Math.sin((arrow_rad)) * nArrowLen;
	var arrow_pos_perp_rad = arrow_rad + 90 * Math.PI / 180; // perpendicular to arrow +90 deg
	var arrow_neg_perp_rad = arrow_rad - 90 * Math.PI / 180; // perpendicular to arrow -90 deg

	if( nArrow==2 )
	{
		// black bar with silver center and tip
		for( var nBoxes=0;nBoxes<3;nBoxes++)
		{
			var nHalfBoxWidth = nMaxTickSize / 6;
			ctx.fillStyle = sGray4;
			ctx.strokeStyle = sGray4;
			if( nBoxes==1 )
			{
				// set up second box
				nHalfBoxWidth = nMaxTickSize / 9;
				ctx.fillStyle = sGray1;
				ctx.strokeStyle = sGray1;
				xTip = xc + Math.cos((arrow_rad)) * (nArrowLen-(nMaxTickSize/6-nMaxTickSize/9));
				yTip = yc + Math.sin((arrow_rad)) * (nArrowLen-(nMaxTickSize/6-nMaxTickSize/9));
			} else if( nBoxes==2 )
			{
				nHalfBoxWidth = nMaxTickSize / 12;
				ctx.fillStyle = sGray3;
				ctx.strokeStyle = sGray3;
				xTip = xc + Math.cos((arrow_rad)) * (nArrowLen-nMaxTickSize/2);
				yTip = yc + Math.sin((arrow_rad)) * (nArrowLen-nMaxTickSize/2);				
			}
			
			xSideA = xc + Math.cos(arrow_pos_perp_rad) * nHalfBoxWidth;
			ySideA = yc + Math.sin(arrow_pos_perp_rad) * nHalfBoxWidth;
		
			xSideB = xc + Math.cos(arrow_neg_perp_rad) * nHalfBoxWidth;
			ySideB = yc + Math.sin(arrow_neg_perp_rad) * nHalfBoxWidth;
	
			xTipA = xTip + Math.cos(arrow_pos_perp_rad) * nHalfBoxWidth;
			yTipA = yTip + Math.sin(arrow_pos_perp_rad) * nHalfBoxWidth;
		
			xTipB = xTip + Math.cos(arrow_neg_perp_rad) * nHalfBoxWidth;
			yTipB = yTip + Math.sin(arrow_neg_perp_rad) * nHalfBoxWidth;
					
			ctx.save();
			ctx.lineWidth = 1;
			ctx.beginPath();
			ctx.moveTo(xTipB, yTipB);
			ctx.lineTo(xTipA, yTipA);
			ctx.lineTo(xSideA, ySideA);
			ctx.lineTo(xSideB, ySideB);
			ctx.fill();
			ctx.restore();			
		}		
	}  else if( nArrow == 3 )	
	{
		// arrow
		var nHalfBoxWidth = nMaxTickSize / 6;
		ctx.fillStyle = sBlue6;
		ctx.strokeStyle = sBlue6;

		xSideA = xc + Math.cos(arrow_pos_perp_rad) * nHalfBoxWidth;
		ySideA = yc + Math.sin(arrow_pos_perp_rad) * nHalfBoxWidth;
	
		xSideB = xc + Math.cos(arrow_neg_perp_rad) * nHalfBoxWidth;
		ySideB = yc + Math.sin(arrow_neg_perp_rad) * nHalfBoxWidth;

		// calc coords for base of tip
		var xTip2 = xc + Math.cos((arrow_rad)) * (nArrowLen-nMaxTickSize/2);
		var yTip2 = yc + Math.sin((arrow_rad)) * (nArrowLen-nMaxTickSize/2);

		xTipA = xTip2 + Math.cos(arrow_pos_perp_rad) * nHalfBoxWidth*2;
		yTipA = yTip2 + Math.sin(arrow_pos_perp_rad) * nHalfBoxWidth*2;
	
		xTipB = xTip2 + Math.cos(arrow_neg_perp_rad) * nHalfBoxWidth*2;
		yTipB = yTip2 + Math.sin(arrow_neg_perp_rad) * nHalfBoxWidth*2;

		// tip
		ctx.fillStyle = sBlue6;
		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.moveTo(xTip, yTip);
		ctx.lineTo(xTipA, yTipA);
		ctx.lineTo(xTipB, yTipB);
		ctx.fill();
		ctx.restore();	

		// prevent gap in traingle and shaft by overwriting arrow a little
		xTip2 = xc + Math.cos((arrow_rad)) * (nArrowLen-nMaxTickSize/2+1);
		yTip2 = yc + Math.sin((arrow_rad)) * (nArrowLen-nMaxTickSize/2+1);

		xTipA = xTip2 + Math.cos(arrow_pos_perp_rad) * nHalfBoxWidth;
		yTipA = yTip2 + Math.sin(arrow_pos_perp_rad) * nHalfBoxWidth;
	
		xTipB = xTip2 + Math.cos(arrow_neg_perp_rad) * nHalfBoxWidth;
		yTipB = yTip2 + Math.sin(arrow_neg_perp_rad) * nHalfBoxWidth;

		// shaft
		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.moveTo(xTipB, yTipB);
		ctx.lineTo(xTipA, yTipA);
		ctx.lineTo(xSideA, ySideA);
		ctx.lineTo(xSideB, ySideB);
		ctx.fill();
		ctx.restore();
	}
	else
	{
		//default or arrow == undefined
		// simple triangle with 2 shades
		xSideA = xc + Math.cos(arrow_pos_perp_rad) * nMaxTickSize / 4;
		ySideA = yc + Math.sin(arrow_pos_perp_rad) * nMaxTickSize / 4;
	
		xSideB = xc + Math.cos(arrow_neg_perp_rad) * nMaxTickSize / 4;
		ySideB = yc + Math.sin(arrow_neg_perp_rad) * nMaxTickSize / 4;
	
		ctx.fillStyle = sBlue;
		ctx.strokeStyle = sBlue2;
	
		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.moveTo(xTip, yTip);
		ctx.lineTo(xSideA, ySideA);
		ctx.lineTo(xSideB, ySideB);
		ctx.fill();
		ctx.restore();
	
		ctx.fillStyle = sBlue6;
		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.moveTo(xTip, yTip);
		ctx.lineTo(xSideA, ySideA);
		ctx.lineTo((xSideA + xSideB) / 2, (ySideA + ySideB) / 2);
		ctx.fill();
		ctx.restore();	
	}
	// draw center circle for gauge
	var nColor = 100;
	var nColorStep = Math.floor((210 - 100) / nCenterPinRadius);
	var nRadius = nCenterPinRadius*1.1
	for ( var n = nRadius; n > nRadius/2; n = n -1 ) 
	{
		ctx.fillStyle = "rgba(" + nColor + ", " + nColor + ", " + nColor + ", 1)";
		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.arc(xc, yc, n, 0, 2 * Math.PI, 0);
		ctx.fill();
		ctx.restore();
		nColor += nColorStep;
	}
	
	var nDotDistFromCenter = (maxRadius - nOutsideRingThickness / 2);
	var darkgrayDot_rad = NA;
	if (darkgrayDot_rad > NA) {
		var nX = xc + Math.cos(darkgrayDot_rad) * nDotDistFromCenter;
		var nY = yc + Math.sin(darkgrayDot_rad) * nDotDistFromCenter;

		ctx.fillStyle = sGray3;
		ctx.strokeStyle = sGray3;

		ctx.save();
		ctx.lineWidth = 1;
		ctx.beginPath();
		ctx.arc(nX, nY, nDotRadius, 0, 2 * Math.PI, 0);
		ctx.fill();
		ctx.restore();
	}

	// draw at bottom of gauge in 2 lines
	if (ctx.measureText)
		aLabels.push({
			x : xc - (ctx.measureText(sDisplay).width +12)/ 2,
			y : 16+nHeight - (2.8 * nFontHeight + 2),
			c : sTextColorForReading,
			l : sDisplay,
			f : "12px 'arial'"
		});
	// second line of label
	if (ctx.measureText)
		aLabels.push({
			x : xc - (ctx.measureText(sDisplay2).width +32) / 2,
			y : 28+nHeight - 1.8 * nFontHeight,
			l : sDisplay2,
			f : "11px 'arial'"
		});

	// draw optional title
	if( sTitle.length > 0 )
	{
		yFontStart = nHeight / 2.6;
		if (ctx.measureText)
			aLabels.push({
				x : xc - ctx.measureText(sTitle).width / 2,
				y : yFontStart,
				c : sBlue,
				l : sTitle
			});
	}
	// draw all gauge labels last
	ctx.fillStyle = sBlack;
	ctx.strokeStyle = sBlack;
	for ( var i = 0; i < aLabels.length; i++) {
		ctx.save();
		ctx.lineWidth = 1;
		if (typeof (aLabels[i].c) != 'undefined') {
			ctx.strokeStyle = aLabels[i].c;
			ctx.fillStyle = aLabels[i].c;
		} else {
			ctx.strokeStyle = sBlack;
			ctx.fillStyle = sBlack;
		}

		if (typeof (aLabels[i].f) != 'undefined')
			ctx.font = aLabels[i].f;
		else
			ctx.font = sFont;

		ctx.translate(aLabels[i].x, aLabels[i].y);
		if (ctx.fillText)
			ctx.fillText(aLabels[i].l + '', 0, 0);
		ctx.restore();
	}
}

